/*********************************************************************
MODULE:         sub1.c

PROGRAMMERS:    John White and Yaoen Zhang

DESCRIPTION:    It contains all functions used for back face removal.
*******************************************************************/

#include "p2.h"

/******************************************************************
FUNCTION:    vec_subtract
PURPOSE:     calculate the vector between two vertices by subtracting
		the vector's "tail" from its "tip"
PARAMETERS:  v1 - A vertex
	     v2 - A vertex
CALLED BY:   back_face()
******************************************************************/
Vector  vec_subtract(v1, v2)
Vector v1, v2;  /* v1 is tip, v2 is tail */
{
Vector result;

result.x = v1.x - v2.x;
result.y = v1.y - v2.y;
result.z = v1.z - v2.z;

return result;
}

/******************************************************************
FUNCTION:    cross 
PURPOSE:     calculate the cross product of two vectors 
PARAMETERS:  v1 - A vertor
	     v2 - A vertor
CALLED BY:   back_face()
******************************************************************/
Vector cross (v1, v2)
Vector v1, v2;
{
Vector result;

result.x = v1.y * v2.z - v2.y * v1.z;
result.y = v2.x * v1.z - v1.x * v2.z;
result.z = v1.x * v2.y - v2.x * v1.y;

return result;
}

/******************************************************************
FUNCTION:    dot 
PURPOSE:     calculate the inner (dot) product of two vectors 
PARAMETERS:  v1 - A vertor
	     v2 - A vertor
CALLED BY:   back_face()
******************************************************************/
int dot (v1, v2)
Vector v1, v2;
{
return (int) (v1.x * v2.x + v1.y * v2.y + v1.z * v2.z);
}

/******************************************************************
FUNCTION:    back_face
PURPOSE:     Test all faces for visibility and flag those faces 
		which are not visible to the viewer 
PARAMETERS:  copy - a modified version of the world box 
CALLED BY:   starting_box(), x_rotate(), y_rotate(), z_rotate(),
		zoom_in(), zoom_out(), up_move(),  down_move(),
		right_move(), left_move() 
******************************************************************/
void back_face(copy)
Box *copy;
{
int i;
Vector x,y,k;

  for(i=0; i<FACES; i++)  {
    x = vec_subtract((copy->vertices[copy->edges[copy->faces[i][0]].v1])
		, (copy->vertices[copy->edges[copy->faces[i][0]].v2]) );

    y = vec_subtract((copy->vertices[copy->edges[copy->faces[i][1]].v1])			, (copy->vertices[copy->edges[copy->faces[i][1]].v2]) );

   proj_box[i].normal = cross(y,x);

   /* copy point at base of normal */
   proj_box[i].pt = copy->vertices[copy->edges[copy->faces[i][1]].v2];
		 
   if(proj_box[i].normal.z > 0)
	copy->faces[i][0] = INVISIBLE;
  }
}
 


