/*********************************************************************
MODULE:         command.c

PROGRAMMERS:    Yaoen Zhang and John White

DESCRIPTION:    This creates widgets and add appropreate callbacks and
                event handlers: command buttons, rotation
                button, slider button
*******************************************************************/

#include "p2.h"
extern void quit();
extern void slider_mark();
extern void mid_zoomin();
extern void mid_zoomout();
extern void mid_mright();
extern void mid_mleft();
extern void mid_mup();
extern void mid_mdown();
extern xstart_mark();
extern ystart_mark();
extern zstart_mark();
extern move_mark();
extern change_direct();
extern mid_xrotate();
extern mid_yrotate();
extern mid_zrotate();
extern void texture();
extern void orientation();
extern void PopupPrompt();
extern void PopupHelp();
extern void reset();

/*******************************************************************
FUNCTION:       command_button
PURPOSE:        It creates and add callbacks to five command buttons.
PARAMETERS:     w -- base window of toplevel,
                data -- data attribute of the box.
CALLED:         main program.
********************************************************************/
command_button(w,in_data)
  Widget w;
  canvas_data *in_data;
{
    Widget quitbut,helpbut, editbut,textbut,resetbut,orientbut;
    quitbut = XtCreateManagedWidget("quitbut", commandWidgetClass,w, NULL, ZERO);
    XtAddCallback(quitbut, XtNcallback, quit, NULL); 

    helpbut= XtCreateManagedWidget("helpbut", commandWidgetClass,w, NULL, ZERO);
    XtAddCallback(helpbut, XtNcallback, PopupHelp, in_data); 

    orientbut=XtCreateManagedWidget("orientbut", commandWidgetClass,w,NULL,ZERO);
    XtAddCallback(orientbut, XtNcallback, orientation, in_data);

    resetbut= XtCreateManagedWidget("resetbut", commandWidgetClass,w, NULL, ZERO);
    XtAddCallback(resetbut, XtNcallback, reset, in_data); 

    editbut = XtCreateManagedWidget("editbut", commandWidgetClass,w, NULL, ZERO);
    XtAddCallback(editbut, XtNcallback, PopupPrompt, in_data);

    textbut = XtCreateManagedWidget("textbut", commandWidgetClass,w, NULL, ZERO);
    XtAddCallback(textbut, XtNcallback, texture, in_data);
}
/*******************************************************************
FUNCTION:       rotation_button
PURPOSE:        It creates and add callbacks to three rotate buttons.
                Rotation buttons: X axis, Y axis, and Z axis.
PARAMETERS:     w -- base window of toplevel,
                data -- data attribute of the box.
CALLED:         main program.
********************************************************************/
rotation_button(w,in_data) 
  Widget w;
  canvas_data *in_data;
{
    Widget direct,rotatex,rotatey,rotatez;
    direct = XtCreateManagedWidget("direct", toggleWidgetClass, w,NULL,ZERO);
    XtAddCallback(direct, XtNcallback, change_direct, in_data);   
    rotatex = XtCreateManagedWidget("rotatex", commandWidgetClass,w, NULL, ZERO);
    XtAddEventHandler(rotatex,ButtonPressMask,FALSE,mid_xrotate,in_data);
    rotatey = XtCreateManagedWidget("rotatey", commandWidgetClass,w, NULL, ZERO);
    XtAddEventHandler(rotatey,ButtonPressMask,FALSE,mid_yrotate,in_data);
    rotatez = XtCreateManagedWidget("rotatez", commandWidgetClass,w, NULL, ZERO);
    XtAddEventHandler(rotatez,ButtonPressMask,FALSE,mid_zrotate,in_data);

}
/*******************************************************************
FUNCTION:       slider_button
PURPOSE:        It creates and add callbacks to three slider boxs.
                Slider: X axis, Y axis, and Z axis.
PARAMETERS:     w -- base window of toplevel,
                data -- data attribute of the box.
CALLED:         main program.
********************************************************************/
slider_button(box0,box1,box2, data)
  Widget box0,box1,box2;
  canvas_data *data;
{
    XtAddEventHandler(box0,ExposureMask,FALSE,slider_mark,NULL);
    XtAddEventHandler(box0,ButtonPressMask,FALSE,xstart_mark,data);
    XtAddEventHandler(box0,ButtonMotionMask,FALSE,move_mark,data);

    XtAddEventHandler(box1,ExposureMask,FALSE,slider_mark,NULL);
    XtAddEventHandler(box1,ButtonPressMask,FALSE,ystart_mark,data);
    XtAddEventHandler(box1,ButtonMotionMask,FALSE,move_mark,data);

    XtAddEventHandler(box2,ExposureMask,FALSE,slider_mark,NULL);
    XtAddEventHandler(box2,ButtonPressMask,FALSE,zstart_mark,data);
    XtAddEventHandler(box2,ButtonMotionMask,FALSE,move_mark,data);
}
/*******************************************************************
FUNCTION:       Translation_button
PURPOSE:        It creates and add callbacks to translation buttons.
PARAMETERS:     w -- base window of toplevel,
                data -- data attribute of the box.
CALLED:         main program.
********************************************************************/
void translation_button(w, in_data)
  Widget w;
  canvas_data *in_data;
{
    Widget zoom_in, zoom_out, mup, mdown, mleft, mright; 

    zoom_in = XtCreateManagedWidget("zoom_in", commandWidgetClass, w, NULL, ZERO);
    XtAddEventHandler(zoom_in,ButtonPressMask,FALSE,mid_zoomin,in_data);

    zoom_out= XtCreateManagedWidget("zoom_out", commandWidgetClass, w, NULL, ZERO);
    XtAddEventHandler(zoom_out,ButtonPressMask,FALSE,mid_zoomout,in_data);

    mup = XtCreateManagedWidget("mup", commandWidgetClass, w, NULL, ZERO);
    XtAddEventHandler(mup,ButtonPressMask,FALSE,mid_mup,in_data);

    mdown = XtCreateManagedWidget("mdown", commandWidgetClass,w, NULL, ZERO);
    XtAddEventHandler(mdown,ButtonPressMask,FALSE,mid_mdown,in_data);

    mleft = XtCreateManagedWidget("mleft", commandWidgetClass,w, NULL, ZERO);
    XtAddEventHandler(mleft,ButtonPressMask,FALSE,mid_mleft,in_data);

    mright = XtCreateManagedWidget("mright", commandWidgetClass,w, NULL, ZERO);
    XtAddEventHandler(mright,ButtonPressMask,FALSE,mid_mright,in_data);
}


