/*********************************************************************
MODULE:         proc.c

PROGRAMMERS:    John White and Yaoen Zhang

DESCRIPTION:    module containing various functions used in p2.c
  		Includes: global_box(), copy_box(), rotate(), 
		scale(), translate(), transform(), render_box(), 
		init_matrix(), & fix_rotation_matrix().
*****************************************************************/ 

#include "p2.h"

extern Vector vec_subtract();
extern Vector cross();
/**************************************************************
FUNCTION: 	global_box	
PURPOSE:        To initialize the world copy of the box which
		is centered at the origin.  Order of edge and
		vertex indices does matter for back face removal. 
PARAMETERS:     none - This function operates on the world
			box model which is declared GLOBALLY
			as box.	
***************************************************************/

void global_box()
{
/* initialize vertices */
        void copy_box();
	box.vertices[0].x = HALFSIZE;
	box.vertices[0].y = HALFSIZE;
	box.vertices[0].z = HALFSIZE;

	box.vertices[1].x = HALFSIZE;
	box.vertices[1].y = HALFSIZE;
	box.vertices[1].z = -HALFSIZE;

	box.vertices[2].x = -HALFSIZE;
	box.vertices[2].y = HALFSIZE;
	box.vertices[2].z = -HALFSIZE;

	box.vertices[3].x = -HALFSIZE;
	box.vertices[3].y = HALFSIZE;
	box.vertices[3].z = HALFSIZE;

	box.vertices[4].x = -HALFSIZE;
	box.vertices[4].y = -HALFSIZE;
	box.vertices[4].z = HALFSIZE;

	box.vertices[5].x = HALFSIZE;
	box.vertices[5].y = -HALFSIZE;
	box.vertices[5].z = HALFSIZE;

	box.vertices[6].x = HALFSIZE;
	box.vertices[6].y = -HALFSIZE;
	box.vertices[6].z = -HALFSIZE;

	box.vertices[7].x = -HALFSIZE;
	box.vertices[7].y = -HALFSIZE;
	box.vertices[7].z = -HALFSIZE;

/* initialize edges */
	box.edges[0].v1 = 0;
	box.edges[0].v2 = 1;

	box.edges[1].v1 = 2;
	box.edges[1].v2 = 1;

	box.edges[2].v1 = 3;
   	box.edges[2].v2 = 2;

	box.edges[3].v1 = 3;
	box.edges[3].v2 = 0;

   	box.edges[4].v1 = 5;
	box.edges[4].v2 = 0;

	box.edges[5].v1 = 4;
	box.edges[5].v2 = 3;

	box.edges[6].v1 = 7;
	box.edges[6].v2 = 2;

	box.edges[7].v1 = 6;
	box.edges[7].v2 = 1;

	box.edges[8].v1 = 4;
	box.edges[8].v2 = 5;

	box.edges[9].v1 = 4;
	box.edges[9].v2 = 7;

	box.edges[10].v1 = 7;
	box.edges[10].v2 = 6;

	box.edges[11].v1 = 5;
	box.edges[11].v2 = 6;

/* initialize faces */
	box.faces[0][0] = 4;
	box.faces[0][1] = 8;
	box.faces[0][2] = 5;
	box.faces[0][3] = 3;

	box.faces[1][0] = 0;
	box.faces[1][1] = 3;
	box.faces[1][2] = 2;
	box.faces[1][3] = 1;

	box.faces[2][0] = 7;
	box.faces[2][1] = 11;
	box.faces[2][2] = 4;
	box.faces[2][3] = 0;

	box.faces[3][0] = 10;
	box.faces[3][1] = 9;
	box.faces[3][2] = 8;
	box.faces[3][3] = 11;

	box.faces[4][0] = 2;
	box.faces[4][1] = 5;
	box.faces[4][2] = 9;
	box.faces[4][3] = 6;

	box.faces[5][0] = 1;
	box.faces[5][1] = 6;
	box.faces[5][2] = 10;
	box.faces[5][3] = 7;
	/* set up the permanent original box */
        copy_box(&originalbox);
}

/**************************************************************
FUNCTION: 	copy_box	
PURPOSE:        Copies the world box into a transitory Box 
		 structure 
PARAMETERS:     destination - the copy of the box 
**************************************************************/
void copy_box(destination)
Box *destination;
{
  int i, j;

  for(i=0;i<VERTICES;i++)
  {
	destination->vertices[i].x = box.vertices[i].x;
	destination->vertices[i].y = box.vertices[i].y;
	destination->vertices[i].z = box.vertices[i].z;
   }

   for(i=0;i<EDGES;i++)
   	destination->edges[i] = box.edges[i];

   for(i=0;i<FACES;i++)
	for(j=0;j<FACE_VERS;j++)
		destination->faces[i][j] = box.faces[i][j];
}

/**************************************************************
FUNCTION:       rotate  	
PURPOSE:        Multilpy all vertices of a copy of the world
		  box by the rotation matrix 
PARAMETERS:     box_copy - the copy of the box 
**************************************************************/
void rotate(box_copy)
Box 	*box_copy;
{
    int i,v;
    double result[MATRIX_DIM];   /* temp */

    for(v=0;v<VERTICES;v++)
    {
	for(i=0;i<MATRIX_DIM;i++)
         result[i] = (box_copy->vertices[v].x * rotation[i][0]) +
		    (box_copy->vertices[v].y * rotation[i][1]) +
		    (box_copy->vertices[v].z * rotation[i][2]);

      box_copy->vertices[v].x = result[0];
      box_copy->vertices[v].y = result[1];
      box_copy->vertices[v].z = result[2];
     }
}

/**************************************************************
FUNCTION:       scale 
PURPOSE:        Multilpy all vertices of a copy of the world
		  box by a corresponding scale factor maintained
		  in data 
PARAMETERS:     box_copy - the copy of the box 
		data - holds important screen and box information
**************************************************************/
void scale(box_copy, data)
Box 	*box_copy;
canvas_data *data;
{
    int i;

    for(i=0;i<VERTICES;i++)
    {
      box_copy->vertices[i].x *= data->x_scale;
      box_copy->vertices[i].y *= data->y_scale;
      box_copy->vertices[i].z *= data->z_scale;
     }
}

/**************************************************************
FUNCTION:       translation 
PURPOSE:        Add to all vertices of a copy of the world
		  box a corresponding position offset 
PARAMETERS:     box_copy - the copy of the box 
		data - holds important screen and box information
**************************************************************/
void translate(box_copy, data)
Box 	*box_copy;
canvas_data *data;
{
    int i;

    for(i=0;i<VERTICES;i++)
    {
      box_copy->vertices[i].x += data->x_translate;
      box_copy->vertices[i].y += data->y_translate;
      box_copy->vertices[i].z += data->z_translate;
     }
}
/**************************************************************
FUNCTION:	transform
PURPOSE:	To properly orient a copy of the world box
		model through scaling, rotation, and translation.
PARAMETERS:	copy - A copy of the box
		in_data - holds important canvas and box data
***************************************************************/
void transform(copy, in_data)
Box *copy;
canvas_data *in_data;
{
int i;
Vector x,y,k;

rotate(copy);
translate(copy,in_data);

    for(i=0; i<FACES; i++)  {
    x = vec_subtract((copy->vertices[copy->edges[copy->faces[i][0]].v1])
    , (copy->vertices[copy->edges[copy->faces[i][0]].v2]) );
    
    y = vec_subtract((copy->vertices[copy->edges[copy->faces[i][1]].v1])
                , (copy->vertices[copy->edges[copy->faces[i][1]].v2]) );
    
    world_box[i].normal = cross(y,x);
    
    /* copy point at base of normal */
    world_box[i].pt = copy->vertices[copy->edges[copy->faces[i][1]].v2];
    
    }

}

/**************************************************************
FUNCTION: 	proj	
PURPOSE:	To project a copy of the box in world coordinates
		onto the projection plan , Z_PLANE.  
PARAMETERS:	copy - A copy of the box
		in_data - holds important canvas and box data
***************************************************************/
void proj(copy, in_data)
Box *copy;
canvas_data  *in_data;
{
int i;	

for(i=0; i<VERTICES; i++)
   	   {
	      copy->vertices[i].x = (Z_PLANE*copy->vertices[i].x)
				/ copy->vertices[i].z;
	      copy->vertices[i].y = (Z_PLANE*copy->vertices[i].y)
				/ copy->vertices[i].z;
	      copy->vertices[i].z = (copy->vertices[i].z -
				Z_PLANE) / copy->vertices[i].z;
           }
}

/**************************************************************
FUNCTION:       ToScreenCoords  	
PURPOSE:        Converts from 3D projected coordinates to the
			viewport
PARAMETERS:     a - a pointer to a point
		b - a pointer to a point	
***************************************************************/
void ToScreenCoords(a,b)
OrderedPair *a,*b;
{
     a->x =((double) SX) * (a->x - XWMIN) + XVMIN;
     a->y =((double) SY) * (a->y - YWMIN) + YVMIN;

     b->x = ((double) SX) * (b->x - XWMIN) + XVMIN;
     b->y = ((double) SY) * (b->y - YWMIN) + YVMIN;

}
/**************************************************************
FUNCTION:       render_box 
PURPOSE:        renders each visible face of the box   
PARAMETERS:     mesh - a copy of the world box 
		data - holds important screen and box information
***************************************************************/
void render_box(mesh, data)
Box *mesh;
canvas_data *data;
{
   int i,j,count;
   OrderedPair pt1,pt2;

   init_screen_region();
   XSetForeground(XtDisplay(data->target), data->gc, CANVASCOLOR);
   XSetFunction(XtDisplay(data->target),data->gc,GXcopy);
   for(i=0;i<FACES;i++)
      if(mesh->faces[i][0] != INVISIBLE)
      {
	for(j=0,count=0;j<FACE_VERS;j++)	{

        	pt1.x = (mesh->vertices[mesh->edges[mesh->faces[i][j]].v1].x);
		pt1.y = (mesh->vertices[mesh->edges[mesh->faces[i][j]].v1].y);
		pt2.x = (mesh->vertices[mesh->edges[mesh->faces[i][j]].v2].x);
		pt2.y = (mesh->vertices[mesh->edges[mesh->faces[i][j]].v2].y);
		ToScreenCoords(&pt1,&pt2);
		XDrawLine(data->w_display,data->window,data->gc, (int) pt1.x,
			(int) pt1.y, (int) pt2.x, (int) pt2.y); 

		screen_region[i][count].x = (int) pt1.x;
		screen_region[i][count++].y = (int) pt1.y;
		screen_region[i][count].x = (int) pt2.x;
		screen_region[i][count++].y = (int) pt2.y;
	  }
	}

   free(mesh);
   order_screen_region();
}

/**************************************************************
FUNCTION:       init_matrix 
PURPOSE:        initializes the values of a matrix to be zero 
PARAMETERS:     matrix - a matrix
***************************************************************/
  
void init_matrix(matrix)
  Matrix matrix;
{
     int i,j;

     for(i=0;i<MATRIX_DIM;i++)
	for(j=0;j<MATRIX_DIM;j++)
		matrix[i][j] = 0;
}

/**************************************************************
FUNCTION:       fix_rotation_matrix 
PURPOSE:        updates a rotation matrix by right
		  multiplying it by a new matrix
PARAMETERS:     matrix - the new matrix
	        target - the global matrix to be update
			(either rotation or back_rotation)	
***************************************************************/

void fix_rotation_matrix(matrix, target, direction)
  Matrix matrix;
  Matrix target;  /* global rotation or inverse rotation matrix */
  int direction;
{
    Matrix tmp;
    int i,j,k;

    init_matrix(tmp);

    /* right multiply - the standard rotation matrix */
    if (direction == RIGHT) {

    for(i=0;i<MATRIX_DIM;i++)	/* row */
      for(j=0;j<MATRIX_DIM;j++)    /* col */
   	for(k=0;k<MATRIX_DIM;k++)   
		tmp[i][j] += target[i][k]*matrix[k][j];
    }

    /* left mutliply - building an inverse matrix */
    else   {

    for(i=0;i<MATRIX_DIM;i++)	/* row */
      for(j=0;j<MATRIX_DIM;j++)    /* col */
   	for(k=0;k<MATRIX_DIM;k++)   
		tmp[i][j] += matrix[i][k]*target[k][j];
    }

    /* update new_matrix */
    for(i=0;i<MATRIX_DIM;i++)
	for(j=0;j<MATRIX_DIM;j++)
		target[i][j] = tmp[i][j];

}


