/*********************************************************************
MODULE:         sub1.c

PROGRAMMERS:    Yaoen Zhang and John White

DESCRIPTION:    It contains all functions called by main program
                Functions include: init_canvas_data, reset, quit,
                in_range, starting_box, starting_tile.
*******************************************************************/

#include "p2.h"

/*******************************************************************
FUNCTION:	init_canvas_data
PURPOSE:  	initialize pointer to Info structure with information
		concerning widgets to be used in subsequent events
PARAMETERS: 	target -- canvas,    
		w -- base window of toplevel,
		gc1 -- graphics context,
		data -- data attribute of the box.
********************************************************************/
void init_canvas_data(canvas,dialog,labels,tilew,box0,box1,box2,data)
Widget canvas,dialog,labels,tilew,box0,box1,box2;
canvas_data *data;
{

    XGCValues gcv;
    GC gc;
    gcv.function = GXcopy;
    gcv.line_style = LineSolid;
    gcv.fill_style = FillStippled;
    gcv.foreground = CANVASCOLOR;
    gcv.background = 0;

    /* tiling box graphics context */
    gc = XCreateGC(XtDisplay(canvas), XtWindow(canvas),
                 GCFunction | GCForeground | GCBackground
                 | GCLineStyle | GCFillStyle,  &gcv);
  data->target = canvas;
  data->message = dialog;
  data->sliderlabel = labels;
  data->tilewin = tilew;
  data->slider0 = box0;
  data->slider1 = box1;
  data->slider2 = box2;
  data->window = XtWindow(canvas);
  data->gc = gc;
  data->w_display = XtDisplay(canvas); /* records display i.d.*/
  data->currentslider = 0;
  data->currentrotate = 1;
  data->x_scale = 1.0;
  data->y_scale = 1.0;
  data->z_scale = 1.0;
  data->xscale_copy = 1.0; 
  data->yscale_copy = 1.0; 
  data->zscale_copy = 1.0; 
  data->x_translate = 0;
  data->y_translate = 0;
  data->z_translate = 396;
}

/*******************************************************************
FUNCTION:       reset
PURPOSE:        reset the whole program, and start again from the
                beginning.
PARAMETERS:     data -- data attribute of the box.
********************************************************************/
void reset(w,data,client_data)
Widget w;
canvas_data *data;
XtPointer client_data;
{
  void init_rotate_matrix();
  void starting_box();
  extern void global_box();
  extern void xstart_mark();
  extern void ystart_mark();
  extern void zstart_mark();

  data->currentslider = 0;
  data->x_scale = 1.0;
  data->y_scale = 1.0;
  data->z_scale = 1.0;
  data->xscale_copy = 1.0;
  data->yscale_copy = 1.0;
  data->zscale_copy = 1.0;
  data->x_translate = 0;
  data->y_translate = 0;
  data->z_translate = 396;
  /* reset the global box data, and the rotation matrix */
  global_box();
  init_rotate_matrix(rotation);
  init_rotate_matrix(back_rotation);
  XClearWindow(data->w_display,data->window);
  starting_box(w,data,(XtPointer) NULL);
  XClearWindow(data->w_display,XtWindow(data->slider0));
  XClearWindow(data->w_display,XtWindow(data->slider1));
  XClearWindow(data->w_display,XtWindow(data->slider2));
  slider_mark(data->slider0,data,(XtPointer) NULL);
  slider_mark(data->slider1,data,(XtPointer) NULL);
  slider_mark(data->slider2,data,(XtPointer) NULL);
  scale_value(data);
}

/*******************************************************************
FUNCTION:	quit()       
PURPOSE:  	destroy all widget and quit.
PARAMETERS:   	w -- widget, 
		call_data, client_data -- not used. 
********************************************************************/
void quit(w,call_data,client_data)
Widget w;
XtPointer call_data, client_data;
{
XtDestroyApplicationContext(XtWidgetToApplicationContext(w));
exit(0);
}


/*******************************************************************
FUNCTION:	init_rotate_matrix()
PURPOSE:    	This function initializes the values of a newly created matrix
		to the identity matrix 
PARAMETERS:     mat - global matrix to initialize: rotation or 
			back_rotation	
********************************************************************/
void init_rotate_matrix(mat)
Matrix mat;
{ 
int i,j;

for(i=0; i<MATRIX_DIM;i++)
	for(j=0;j<MATRIX_DIM;j++)
	{
		mat[i][j] = ( i == j);
	}
}


/*******************************************************************
FUNCTION:	starting_box()
PURPOSE:    	This draws the starting box on canvas.
PARAMETERS:	none
********************************************************************/
void starting_box(widget, call_data, client_data)
Widget widget;
canvas_data	*call_data;
XtPointer	client_data;
{
extern render_box(), copy_box(), transform(), proj();
Box *mesh;

mesh = (Box *) malloc (sizeof(Box));

copy_box(mesh);
transform(mesh,call_data);
proj(mesh,call_data);
back_face(mesh);
render_box(mesh,call_data);		/* draw the box, and remove event handler */
XtRemoveEventHandler(call_data->target, ExposureMask, FALSE, starting_box,
			     call_data);
}
/************************************************************************
FUNCTION:       in_range
PURPOSE:        To see if a point in world coordinates in within the 
		world boundary
PARAMETERS:     point - a vector in 3-space
***************************************************************************/
int in_range(copy,data)
Box *copy;
canvas_data *data;
{
int i, result = 1;
double zshort, zlong;

zshort = (double) (ZNEAR - Z_PLANE)/ZNEAR;
zlong = (double) (ZFAR - Z_PLANE)/ZFAR;
for(i=0; i<VERTICES; i++)
        if (
                copy->vertices[i].x > XWMAX ||
                copy->vertices[i].x < XWMIN ||
                copy->vertices[i].y > YWMAX ||
                copy->vertices[i].y < YWMIN ||
                copy->vertices[i].z > zlong ||
                copy->vertices[i].z < zshort )
                {
                        put_message(data,"Cannot move out of bounds");
                        result = 0;
                        break;
                }
return result;
}
/*******************************************************************
FUNCTION:	starting_tile()
PURPOSE:    	This draws the starting box on canvas.
PARAMETERS:	none
********************************************************************/
void starting_tile(widget, call_data, client_data)
Widget widget;
canvas_data	*call_data;
XtPointer	client_data;
{
extern draw_maps();
draw_maps(widget, call_data, (XtPointer)NULL);
}   
